//+------------------------------------------------------------------+
//|                                        stochastic volatility.mq4 |
//|                                                                  |
//| made after the original work of Francesco G. Cavasino            |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  6
#property indicator_color1   DeepSkyBlue
#property indicator_color2   Red
#property indicator_color3   Red
#property indicator_color4   LimeGreen
#property indicator_color5   Yellow
#property indicator_color6   PaleVioletRed
#property indicator_width1   2
#property indicator_style3   STYLE_DOT
#property indicator_style4   STYLE_DOT

//
//
//
//
//

extern int    Length             =   5;
extern int    Slowing            =   3;
extern int    Signal             =   3;
extern int    BandsLength        = 126;
extern double BandsDeviation     =   1;
extern int    Price              = PRICE_CLOSE;
extern bool   OriginalStoch      = true;
extern bool   OriginalVolatility = true;

//
//
//
//
//

double stvBuffer[];
double sigBuffer[];
double bbuBuffer[];
double bbdBuffer[];
double entBuffer[];
double extBuffer[];
double logBuffer[];
double volBuffer[];
double wrkBuffer[][2];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,stvBuffer);
   SetIndexBuffer(1,sigBuffer);
   SetIndexBuffer(2,bbuBuffer);
   SetIndexBuffer(3,bbdBuffer);
   SetIndexBuffer(4,entBuffer); SetIndexStyle(4,DRAW_ARROW); SetIndexArrow(4,108);
   SetIndexBuffer(5,extBuffer); SetIndexStyle(5,DRAW_ARROW); SetIndexArrow(5,108);
   SetIndexBuffer(6,logBuffer);
   SetIndexBuffer(7,volBuffer);
   
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define __hi 0
#define __lo 1

//
//
//
//
//

double prcBuffer[];

int start()
{
   double alpha = 2.0 / (1.0 + Slowing);
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(wrkBuffer,0)!=Bars) { ArrayResize(wrkBuffer,Bars); ArrayResize(prcBuffer,Bars); }
         
   //
   //
   //
   //
   //
            
   for(i=limit,r=Bars-i-1; i>=0; i--,r++)
   {
      prcBuffer[r] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      
         double vol1 = volatility(prcBuffer,logBuffer,10,i,r)*3.0;
         double vol2 = volatility(prcBuffer,logBuffer,21,i,r)*3.0;
         double vol3 = volatility(prcBuffer,logBuffer,34,i,r)*2.0;
         double vol4 = volatility(prcBuffer,logBuffer,55,i,r);
         double vol5 = volatility(prcBuffer,logBuffer,89,i,r);
         
            volBuffer[i]=(vol1+vol2+vol3+vol4+vol5);
   
      //
      //
      //
      //
      //
        
         double hi = volBuffer[i];
         double lo = volBuffer[i];
            for (int k=1; k<Length && (i+k)<(Bars-1); k++)
            {
               hi = MathMax(hi,volBuffer[i+k]);
               lo = MathMin(lo,volBuffer[i+k]);
            }
            wrkBuffer[r][__hi] = hi;
            wrkBuffer[r][__lo] = lo;
            
         
      //
      //
      //
      //
      //

         double stoc = 0;               
            if (OriginalStoch)
            {
               double sumRng = 0;
               double sumLow = 0;
               for (k=0; k<Slowing; k++)
               {
                  sumRng += wrkBuffer[r-k][__hi]-wrkBuffer[r-k][__lo];
                  sumLow += volBuffer[i+k]      -wrkBuffer[r-k][__lo];
               }
               if (sumRng!=0) stoc = 100*sumLow/sumRng;
            }
            else
            {
               double stov = 0;
               if (hi!=lo) stov = 100*(volBuffer[i]-lo)/(hi-lo);
                           stoc = MathMax(MathMin(stvBuffer[i+1]+alpha*(stov-stvBuffer[i+1]),100),0);
            }            
      
         //
         //
         //
         //
         //
                  
         stvBuffer[i] = stoc;
         sigBuffer[i] = iSma(stvBuffer,Signal,i);

      //
      //
      //
      //
      //
      
        double average   = iSma(stvBuffer,BandsLength,i);
        double deviation = iDeviation(stvBuffer,BandsLength,average,i);
            bbuBuffer[i] = average+deviation+BandsDeviation;
            bbdBuffer[i] = average-deviation+BandsDeviation;
            entBuffer[i] = EMPTY_VALUE;
            extBuffer[i] = EMPTY_VALUE;
            if (stvBuffer[i] < bbdBuffer[i]) entBuffer[i] = stvBuffer[i];
            if (stvBuffer[i] > bbuBuffer[i]) extBuffer[i] = stvBuffer[i];
   }
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double volatility(double& array[], double& destArray[], int length, int i, int r)
{
   if (array[r-1]!=0)
         destArray[i] = MathLog(array[r]/array[r-1]);
   else  destArray[i] = 0;

   if (i>(Bars-length)) return(0);
   
   double average   = iSma(destArray,length,i);
   double deviation = iDeviation(destArray,length,average,i);
   if (OriginalVolatility)
         return((average+deviation)*MathSqrt(252));
   else  return(deviation);
}

//
//
//
//
//

double iSma(double& array[], int period, int pos)
{
   double sum = array[pos];
      for(int i=1; i<period; i++) sum += array[pos+i];
   return(sum/period);     
}

double iDeviation(double& array[], int period, double dMA, int pos)
{
   double dSum = 0;
      for(int i=0; i<period; i++) dSum += (array[pos+i]-dMA)*(array[pos+i]-dMA);
   return(MathSqrt(dSum/period));
}